package com.gk_software.core.maven.mojo;

import org.apache.logging.log4j.Logger;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.util.FileUtils;
import com.gk_software.core.maven.utils.CustomConfigurationFactory;
import com.gk_software.core.maven.utils.ModificationHelper;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;

@Mojo(name = "list-changed-do", defaultPhase = LifecyclePhase.INSTALL)
public class ChangedDO extends AbstractExecution {

    @Parameter(required = false, defaultValue = "")
    private String excludes;

    private static Logger logger;
    private String[] regExExcludes = null;

    @Override
    public void execute() throws MojoExecutionException, MojoFailureException {
        long startTime = System.nanoTime();
        getLog().warn("'list-changed-do' is deprecated mojo");
        getLog().warn("Please use 'check-folder' execution with includes set to '*.do'");
        getLog().warn("'check-folder' is implemented from version 1.7.");
        logger = CustomConfigurationFactory.getLogger(this.getOutput() + "list-changed-do");
        if(!this.checkProductData()){
            return;
        }

        if (this.excludes != null) this.regExExcludes = ModificationHelper.excludesIntoPath(excludes);


        LinkedList<File> oldProductFiles = ModificationHelper.filterFiles(this.getOldVersionFiles(),
                ".do");
        if (oldProductFiles.size() == 0) {
            getLog().info("There are no old product '.do' files");
            logger.info("There are no old product '.do' files");
            return;
        }
        getLog().info("Number of old version do files: " + oldProductFiles.size());

        LinkedList<File> newProductFiles = ModificationHelper.filterFiles(this.getNewVersionFiles(),
                ".do");
        if (newProductFiles.size() == 0) {
            getLog().info("There are no new product '.do' files");
            logger.info("There are no new product '.do' files");
            return;
        }
        getLog().info("Number of new version '.do' files: " + newProductFiles.size());

        int changedFiles = findOwFilesBetweenVersions(oldProductFiles, newProductFiles);

        if (changedFiles == 0) {
            getLog().info("There are no changes among '.do' files");
            logger.info("There are no changes among '.do' files");
        } else {
            getLog().info("There are " + changedFiles + " changed files among '.do' product files");
        }
        long endTime = (System.nanoTime() - startTime) / 1000000;
        getLog().info("Execution time: " + endTime+"ms");
    }

    private int findOwFilesBetweenVersions(LinkedList<File> oldProductFiles, LinkedList<File> newProductFiles) {
        boolean foundFilesOnSamePath = false;
        int changedFiles = 0;
        for (int i = 0; i < oldProductFiles.size(); i++) {
            File oldFile = oldProductFiles.get(i);
            String relativeOldPath = getPathFromString(oldFile.getAbsolutePath(), this.getOldVersionPath());
            for (int j = 0; j < newProductFiles.size(); j++) {
                File newFile = newProductFiles.get(j);
                String relativeNewPath = getPathFromString(newFile.getAbsolutePath(), this.getNewVersionPath());
                if (relativeNewPath.equals(relativeOldPath)) {
                    foundFilesOnSamePath = true;
                    try {
                        if (!FileUtils.contentEquals(oldFile, newFile)) {
                            if (!ModificationHelper.isAmongExcludes(regExExcludes,relativeOldPath)){
                                logger.info("changed do file between versions: " + relativeOldPath);
                                changedFiles++;
                            }
                        }
                        oldProductFiles.remove(i);
                        newProductFiles.remove(j);
                        j--;
                        i--;
                        break;
                    } catch (IOException e) {
                        logger.error("Error occurred while comparing files on path: " + relativeOldPath);
                        getLog().error("Error occurred while comparing files on path: " + relativeOldPath);
                    }
                }
            }
            if (!foundFilesOnSamePath) {
                if (!ModificationHelper.isAmongExcludes(regExExcludes,relativeOldPath)){
                    logger.info("changed do file between versions: " + relativeOldPath);
                    changedFiles++;
                }
            } else {
                foundFilesOnSamePath = false;
            }
        }
        return changedFiles;
    }

    private String getPathFromString(String absolutePath, String splitFrom) {
        return absolutePath.replace(splitFrom, "");
    }


}
