package com.gk_software.core.maven.mojo;

import com.gk_software.core.maven.utils.CustomConfigurationFactory;
import com.gk_software.core.maven.utils.ModificationHelper;
import org.apache.logging.log4j.Logger;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.util.FileUtils;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

@Mojo(name = "check-folder", defaultPhase = LifecyclePhase.INSTALL)
public class CheckFolder extends AbstractExecution {

    @Parameter(required = false, defaultValue = "")
    private String includes;

    @Parameter(required = false, defaultValue = "")
    private String excludes;

    private static Logger logger;

    @Override
    public void execute() throws MojoExecutionException, MojoFailureException {
        long startTime = System.nanoTime();
        logger = CustomConfigurationFactory.getLogger(this.getOutput() + "check-folder");
        if (!this.checkProductData()) {
            return;
        }
        LinkedList<File> oldVersionFiles = this.getOldVersionFiles(includes, excludes);
        LinkedList<File> newVersionFiles = this.getNewVersionFiles(includes, excludes);

        if (oldVersionFiles.size() == 0) {
            getLog().error("There were not found any files in old version");
            return;
        }

        if (newVersionFiles.size() == 0) {
            getLog().error("There were not found any files in new version");
            return;
        }

        getLog().info("Old version files found: " + oldVersionFiles.size());
        getLog().info("New version files found: " + newVersionFiles.size());


        if (includes != null) getLog().info("Set includes: "+ includes);
        if (excludes != null) getLog().info("Set excludes: "+ excludes);
        int changedFiles = findOwFilesBetweenVersions(oldVersionFiles, newVersionFiles);
        getLog().info("There are " + changedFiles + " changed files in product folders");

        long endTime = (System.nanoTime() - startTime) / 1000000;
        getLog().info("Execution time: " + endTime+"ms");
    }

    private int findOwFilesBetweenVersions(LinkedList<File> oldProductFiles, LinkedList<File> newProductFiles) {
        int changedFiles = 0;

        Hashtable<String, File> oldF = getFilesOnRelativePath(oldProductFiles, this.getOldVersionPath());
        Hashtable<String, File> newF = getFilesOnRelativePath(newProductFiles, this.getNewVersionPath());
        logger.info("Changed files between versions: ");

        Iterator<Map.Entry<String, File>> it = oldF.entrySet().iterator();
        while (it.hasNext()){
            Map.Entry<String, File> entry = it.next();
            if (newF.get(entry.getKey()) != null){
                try {
                    if (!FileUtils.contentEquals(entry.getValue(), newF.get(entry.getKey()))){
                        logger.info("\t -> " + entry.getKey());
                        changedFiles++;
                    }
                } catch (IOException e) {
                    logger.error("Error occurred while comparing files on path: " + entry.getKey());
                    getLog().error("Error occurred while comparing files on path: " + entry.getKey());
                }

                newF.remove(entry.getKey());
                it.remove();
            }
        }


        getLog().info("There are " + oldF.keySet().size() + " deleted files");
        getLog().info("There are " + newF.keySet().size() + " added files");
        logger.info("\nDeleted files between versions: ");
        for(String path: oldF.keySet()){
            logger.info("\t -> " + path);
        }
        logger.info("\nAdded files between versions: ");
        for(String path: newF.keySet()){
            logger.info("\t -> " + path);
        }

        return changedFiles;
    }

    private Hashtable<String, File> getFilesOnRelativePath(LinkedList<File> files, String prefix) {
        Hashtable<String, File> newFiles = new Hashtable<   >();
        for(File f: files){
            String relativePath = ModificationHelper.getPathFromString(f.getAbsolutePath(), prefix);
            newFiles.put(relativePath, f);
        }
        return newFiles;
    }

}
