package com.gk_software.core.maven.mojo;

import org.apache.logging.log4j.Logger;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.plugins.annotations.LifecyclePhase;
import org.apache.maven.plugins.annotations.Mojo;
import org.apache.maven.plugins.annotations.Parameter;
import org.codehaus.plexus.util.FileUtils;
import com.gk_software.core.maven.utils.CustomConfigurationFactory;
import com.gk_software.core.maven.utils.ModificationHelper;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

@Mojo(name = "check-xml-layout", defaultPhase = LifecyclePhase.INSTALL)
public class CheckXmlLayout extends AbstractExecution {
    @Parameter(property = "layouts")
    private List<String> layouts;

    private static final String xmlSuffix = ".xml";

    private static Logger logger;

    @Override
    public void execute() throws MojoExecutionException, MojoFailureException {
        long startTime = System.nanoTime();
        logger = CustomConfigurationFactory.getLogger(this.getOutput() + "check-xml-layout");
        if (!this.checkProductData()) {
            return;
        }
        if (this.layouts.size() == 0) {
            getLog().error("Layouts were not found in configuration");
            return;
        }
        LinkedList<String> layoutPaths = getLayouts();
        compareLayouts(layoutPaths);

        long endTime = (System.nanoTime() - startTime) / 1000000;
        getLog().info("Execution time: " + endTime+"ms");
    }

    private void compareLayouts(LinkedList<String> layoutPaths) {
        int owFound = 0;
        for (String path : layoutPaths) {
            File oldLayout = new File(this.getOldVersionPath() + path);
            File newLayout = new File(this.getNewVersionPath() + path);
            if (!oldLayout.exists()) {
                logger.warn("Xml layout (" + ModificationHelper.getRelativePath(oldLayout.getAbsolutePath(), this.getOldVersionPath()) + ") was not found in old product");
                getLog().warn("Xml layout (" + ModificationHelper.getRelativePath(oldLayout.getAbsolutePath(), this.getOldVersionPath()) + ") was not found in old product");
            } else if (!newLayout.exists()) {
                logger.warn("Xml layout (" + ModificationHelper.getRelativePath(newLayout.getAbsolutePath(), this.getNewVersionPath()) + ") was not found in new product");
                getLog().warn("Xml layout (" + ModificationHelper.getRelativePath(newLayout.getAbsolutePath(), this.getNewVersionPath()) + ") was not found in new product");
            } else {
                try {
                    if (!FileUtils.contentEquals(oldLayout, newLayout)) {
                        logger.info("Changed layout found on path: '" + ModificationHelper.getRelativePath(oldLayout.getAbsolutePath(), this.getOldVersionPath()) + "'");
                        owFound++;
                    }
                } catch (IOException e) {
                    logger.error("Error occurred while comparing files on path: '" + ModificationHelper.getRelativePath(oldLayout.getAbsolutePath(), this.getOldVersionPath()) + "'");
                    getLog().error("Error occurred while comparing files on path: '" + ModificationHelper.getRelativePath(oldLayout.getAbsolutePath(), this.getOldVersionPath()) + "'");
                }
            }
        }
        if (owFound == 0) {
            logger.info("No changed layouts found");
            getLog().info("No changed layouts found");
        }
    }

    private LinkedList<String> getLayouts() {
        LinkedList<String> layoutPaths = new LinkedList<String>();
        for (int i = 0; i < layouts.size(); i++) {
            String layout = layouts.get(i);
            if (layout.contains(".")){
                layout = layout.replace(".", "/");
            }
            if (!layout.endsWith(xmlSuffix)) {
                layout += xmlSuffix;
            }
            layout = "/" + layout;
            layoutPaths.add(layout);
        }
        return layoutPaths;
    }

}
