package com.gk_software.core.maven.parser;

import java.io.File;
import java.io.IOException;
import java.util.LinkedList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DoinFile extends AbstractParser {

    private static final Pattern CLASS_PATTERN = Pattern.compile("((?:collection))\\s([a-zA-Z0-9_\\-.]+)");
    private static final Pattern EXTENDS_PATTERN = Pattern.compile("(?:collection)\\s([a-zA-Z0-9_\\-.]+).(?::).([a-zA-Z0-9_\\-.]+)");
    private static final String SUFFIX = "_DO.do";

    private LinkedList<String> references;

    public DoinFile(File file) throws IOException {
        super(file);
    }

    public LinkedList<String> getReferences() {
        return references;
    }

    @Override
    protected int parseData(String line){
        Matcher m = getImportPattern().matcher(line);
        if (m.find()){
            String tmpImport = m.group(1);
            imports.add(tmpImport);
            return CONTINUE_READING;
        }
        m = getClassPattern().matcher(line);
        if (m.find()) {
            m = extendsPattern().matcher(line);
            if (m.find()){
                String extendedClass = getSuperClassFromLine(m.group(0));
                superClass = addSuperClassPath(extendedClass);
                if (this.references == null){
                    this.references = new LinkedList<>();
                }
                this.references.add(superClass);
            }
            return CONTINUE_READING;
        }
        return CONTINUE_READING;
    }

    @Override
    protected String addSuperClassPath(String superClass) {
        return superClass + getSuffix();
    }

    @Override
    protected String getSuperClassFromLine(String group) {
        String[] tokens = group.split(":");
        return tokens[1].trim();
    }

    @Override
    protected String getSuffix() {
        return SUFFIX;
    }

    @Override
    protected Pattern extendsPattern() {
        return EXTENDS_PATTERN;
    }

    @Override
    protected Pattern getClassPattern() {
        return CLASS_PATTERN;
    }
}
