package com.gk_software.core.maven.parser;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

class JavaFileTest {

    @Test
    void parseFileWithProductReferenceAndSuperclass() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File javaFileWithProductReference = new File(classLoader.getResource("java/JavaFileWithProductReference.java").getFile());

        String expectedSuperClass = "\\com\\gk_software\\pos\\api\\service\\client\\scale\\SuperClass.java";
        List<String> expectedImports = new LinkedList<>();
        expectedImports.add("org.springframework.beans.factory.InitializingBean");
        expectedImports.add("com.gk_software.pos.api.service.client.scale.SuperClass");
        List<String> expectedReferences = new LinkedList<>();
        expectedReferences.add("\\com\\gk_software\\pos\\service\\client\\scale\\ReferencedClass.java");

        JavaFile javaFile = new JavaFile(javaFileWithProductReference);
        assertAll(() -> assertTrue(javaFile.isProductReference()),
                () -> assertEquals(expectedSuperClass, javaFile.getSuperClass()),
                () -> assertIterableEquals(expectedImports, javaFile.getImports()),
                () -> assertIterableEquals(expectedReferences, javaFile.getReferences()));
    }

    @Test
    void parseFileWithProductReference() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File javaFileWithProductReference = new File(classLoader.getResource("java/WithoutSuperClassWithReference.java").getFile());

        String expectedSuperClass = "";
        List<String> expectedReferences = new LinkedList<>();
        expectedReferences.add("\\com\\gk_software\\pos\\service\\client\\scale\\ReferencedClass.java");

        JavaFile javaFile = new JavaFile(javaFileWithProductReference);
        assertAll(() -> assertTrue(javaFile.isProductReference()),
                () -> assertEquals(expectedSuperClass, javaFile.getSuperClass()),
                () -> assertIterableEquals(Collections.emptyList(), javaFile.getImports()),
                () -> assertIterableEquals(expectedReferences, javaFile.getReferences()));
    }

    @Test
    void checkThatReadingEndsWithFileHeader() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File file = new File(classLoader.getResource("java/ReadingCheck.java").getFile());
        JavaFile javaFile = new JavaFile(file);

        assertAll(() -> assertEquals("", javaFile.getSuperClass()),
                () -> assertIterableEquals(Collections.emptyList(), javaFile.getImports()));
    }

}