package com.gk_software.core.maven.parser;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import static org.junit.jupiter.api.Assertions.*;

class PromoFileTest {

    @Test
    void getSuperClassFromLine() throws IOException {
        String lineIncludingInheritance = "Sticky Process EntityName_EXT specializes Superclass";
        ClassLoader classLoader = getClass().getClassLoader();
        File file = new File(classLoader.getResource("promo/EntityName_EXT.promo").getFile());
        PromoFile promoFile = new PromoFile(file);
        String expectedSuperClassName = "Superclass";

        assertEquals(expectedSuperClassName, promoFile.getSuperClassFromLine(lineIncludingInheritance));
    }

    @Test
    void parsePromoWithSuperClass() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File withSuperClass = new File(classLoader.getResource("promo/EntityName_EXT.promo").getFile());
        PromoFile promoFile = new PromoFile(withSuperClass);

        String expectedSuperClass = "\\processModels\\tpos\\common\\registration\\Superclass.promo";
        List<String> expectedImports = new LinkedList<>();
        expectedImports.add("processModels.tpos.common.registration.Superclass");

        assertAll(() -> assertEquals(expectedSuperClass, promoFile.getSuperClass()),
                () -> assertIterableEquals(expectedImports, promoFile.getImports()));
    }

    @Test
    void parsePromoWithoutSuperClass() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File withoutSuperClass = new File(classLoader.getResource("promo/WithoutSuperClass_EXT.promo").getFile());
        PromoFile promoFile = new PromoFile(withoutSuperClass);

        assertAll(() -> assertNull(promoFile.getSuperClass()),
                () -> assertIterableEquals(Collections.emptyList(), promoFile.getImports()));
    }

    @Test
    void checkThatReadingEndsWithFileHeader() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File withoutSuperClass = new File(classLoader.getResource("promo/ReadingCheck.promo").getFile());
        PromoFile promoFile = new PromoFile(withoutSuperClass);

        assertAll(() -> assertNull(promoFile.getSuperClass()),
                () -> assertIterableEquals(Collections.emptyList(), promoFile.getImports()));
    }
}