package com.gk_software.core.maven.parser;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.io.IOException;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

class PromoJFileTest {

    @Test
    void getSuperClassFromLine() throws IOException {
        String lineIncludingInheritance = "Process EntityName_EXT specializes Superclass";
        ClassLoader classLoader = getClass().getClassLoader();
        File file = new File(classLoader.getResource("promoj/EntityName_EXT.promoj").getFile());
        PromoJFile promoJFile = new PromoJFile(file);
        String expectedSuperClassName = "Superclass";

        assertEquals(expectedSuperClassName, promoJFile.getSuperClassFromLine(lineIncludingInheritance));
    }

    @Test
    void parsePromoWithSuperClass() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File withSuperClass = new File(classLoader.getResource("promoj/EntityName_EXT.promoj").getFile());
        PromoJFile promoJFile = new PromoJFile(withSuperClass);

        String expectedSuperClass = "\\processModels\\tpos\\simple\\registration\\Superclass.promoj";
        List<String> expectedImports = new LinkedList<>();
        expectedImports.add("processModels.tpos.simple.registration.Superclass");

        assertAll(() -> assertEquals(expectedSuperClass, promoJFile.getSuperClass()),
                () -> assertIterableEquals(expectedImports, promoJFile.getImports()));
    }

    @Test
    void parsePromoWithoutSuperClass() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File withoutSuperClass = new File(classLoader.getResource("promoj/WithoutSuperClass_EXT.promoj").getFile());
        PromoJFile promoJFile = new PromoJFile(withoutSuperClass);

        assertAll(() -> assertNull(promoJFile.getSuperClass()),
                () -> assertIterableEquals(Collections.emptyList(), promoJFile.getImports()));
    }

    @Test
    void checkThatReadingEndsWithFileHeader() throws IOException {
        ClassLoader classLoader = getClass().getClassLoader();
        File withoutSuperClass = new File(classLoader.getResource("promoj/ReadingCheck.promoj").getFile());
        PromoFile promoFile = new PromoFile(withoutSuperClass);

        assertAll(() -> assertNull(promoFile.getSuperClass()),
                () -> assertIterableEquals(Collections.emptyList(), promoFile.getImports()));
    }

}