package com.gk_software.core.maven.utils;

import org.junit.jupiter.api.Test;

import java.io.File;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Hashtable;
import java.util.LinkedList;

import static org.junit.jupiter.api.Assertions.*;

class ModificationHelperTest {

    @Test
    void getRelativePath() {
        String fileFullPath = "C:/Users/BP/Module/src/main/java/com/gk_software/cst/pos/flow/file.java";
        String fileDirectory = "C:/Users/BP/Module/";

        Path expectedResult = Paths.get("src/main/java/com/gk_software/cst/pos/flow/file.java");

        assertEquals(expectedResult, ModificationHelper.getRelativePath(fileFullPath, fileDirectory));
    }

    @Test
    void isAmongExcludesAndExcludesNull_shouldReturnFalse() {
        String path = "path";
        assertFalse(ModificationHelper.isAmongExcludes(null, path));
    }

    @Test
    void isAmongExcludesAndPathInExcludes_shouldReturnTrue() {
        String[] excludes = {"com/gk_software/cst/pos/flow/file.java","com/gk_software/cst/"};
        String path = "src/main/java/com/gk_software/cst/pos/flow/file.java";

        assertTrue(ModificationHelper.isAmongExcludes(excludes, path));
    }

    @Test
    void isNotAmongExcludes_shouldReturnFalse() {
        String[] excludes = {"com/gk_software/cst/pos/flow/file.java","com/gk_software/cst/"};
        String path = "src/file.java";

        assertFalse(ModificationHelper.isAmongExcludes(excludes, path));
    }

    @Test
    void excludesIntoPath() {
        String exclude = "processModels.tpos.simple.item.ItemInfo, processModels.tpos.exam.item.ItemInfo";
        String[] expectedResult = {"\\processModels\\tpos\\simple\\item\\ItemInfo", "\\processModels\\tpos\\exam\\item\\ItemInfo"};

        assertArrayEquals(expectedResult, ModificationHelper.excludesIntoPath(exclude));
    }

    @Test
    void strToRegex() {
        String parameter = "*.java, *.do, \\processModels\\tpos\\simple\\item\\ItemInfo";
        String[] expectedResult = {".java", ".do", "\\processModels\\tpos\\simple\\item\\ItemInfo"};

        assertArrayEquals(expectedResult, ModificationHelper.strToRegex(parameter));
    }

    @Test
    void getPathFromPackage() {
        String packagePath = "processModels.tpos.simple.item.ItemInfo";
        String expectedResult = "\\processModels\\tpos\\simple\\item\\ItemInfo";

        assertEquals(expectedResult, ModificationHelper.getPathFromPackage(packagePath));
    }

    @Test
    void testGetPathFromPackage() {
        String packagePath = "processModels.tpos.simple.item.ItemInfo";
        String suffix = ".java";
        String expectedResult = "\\processModels\\tpos\\simple\\item\\ItemInfo.java";

        assertEquals(expectedResult, ModificationHelper.getPathFromPackage(packagePath, suffix));
    }

    @Test
    void testGetPathFromPackage1() {
        String packagePath = "processModels.tpos.simple.item.ItemInfo";
        String prefix = "C:";
        String suffix = "java";
        String expectedResult = "C:\\processModels\\tpos\\simple\\item\\ItemInfo.java";

        assertEquals(expectedResult, ModificationHelper.getPathFromPackage(packagePath,prefix, suffix));
    }

    @Test
    void filterFiles() {
        LinkedList<File> files = new LinkedList<>();
        files.add(new File("test\\package\\filename.java"));
        files.add(new File("test\\package\\filename.do"));
        String suffix = ".java";

        LinkedList<File> expectedResult = new LinkedList<>();
        expectedResult.add(new File("test\\package\\filename.java"));

        assertIterableEquals(expectedResult, ModificationHelper.filterFiles(files, suffix));
    }

    @Test
    void testFilterFiles() {
        LinkedList<File> files = new LinkedList<>();
        files.add(new File("test\\package\\filename.java"));
        files.add(new File("test\\package\\filename.do"));
        files.add(new File("test\\package\\filename.doin"));
        String[] suffix = {".java", ".doin"};

        LinkedList<File> expectedResult = new LinkedList<>();
        expectedResult.add(new File("test\\package\\filename.java"));
        expectedResult.add(new File("test\\package\\filename.doin"));

        assertIterableEquals(expectedResult, ModificationHelper.filterFiles(files, suffix));
    }

    @Test
    void testFilterFiles1() {
    }

    @Test
    void getPathFromString() {
        String fileFullPath = "C:/Users/BP/Module/src/main/java/com/gk_software/cst/pos/flow/file.java";
        String fileDirectory = "C:/Users/BP/Module/";
        String expectedResult = "src/main/java/com/gk_software/cst/pos/flow/file.java";

        assertEquals(expectedResult, ModificationHelper.getPathFromString(fileFullPath, fileDirectory));
    }

    @Test
    void getLineFromHashTable() {
        String key = "\\javax\\swing\\JLabel.java";
        String fileRelativePath = "\\com\\gk_software\\pos\\ui\\swing\\component\\label\\GkPosStateInfoLabel.java";
        String message = "file not found";
        ClassData classData = new ClassData(message, fileRelativePath);
        String expectedResult = "\\javax\\swing\\JLabel.java - file not found -->\n" +
                " \t\\com\\gk_software\\pos\\ui\\swing\\component\\label\\GkPosStateInfoLabel.java";

        assertEquals(expectedResult, ModificationHelper.getLineFromHashTable(key, classData));
    }

    @Test
    void addToHashTable() {
        Hashtable<String, ClassData> hashtable = new Hashtable<>();
        String key = "\\javax\\swing\\JLabel.java";
        String fileRelativePath = "\\com\\gk_software\\pos\\ui\\swing\\component\\label\\GkPosStateInfoLabel.java";
        String message = "file not found";

        ModificationHelper.addToHashTable(hashtable, key, fileRelativePath, message);
        assertTrue(hashtable.containsKey(key));
        ModificationHelper.addToHashTable(hashtable, key, fileRelativePath, message);
        assertEquals(2, hashtable.get(key).getClasses().size());
    }
}